#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 12-Jul-2003
#################################################################################
#
# Configure Puppeteer - Script part of Ethereal Realms, designed to configure
#                       a puppeteers settings
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);                                    # MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use eRealms::Audit;						# Auditing handle
use eRealms::Database;						# Database handler
use eRealms::Login;						# Login functionality
use eRealms::Option;						# Option handlings
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table control
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my %config;							# Configuration tag
my %login;							# Login handler

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param    = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Fetch hash
	$database->GetHashConfiguration(\%config);
	$database->GetHashLogin(\%login);

	# Template handle
	$tmpl = new eRealms::Template(\%login);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login    = new eRealms::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Chat preferences handling
		if (defined($param->{'SETPREFS'}))
		{ ChatHandler($database, $param, \%login); }

		# Information handling
		if (defined($param->{'OLDPASS'}) || defined($param->{'EMAIL'}))
		{ InformationHandler($database, $cgi, $param, \%config); }


		# Initial screen	
		InitialDisplay($database, $cgi, $param, $tmpl, \%config, \%login);
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Chat Prefereces Handler
#
# Takes care of the database updates when one wants to change chat preferences
# this is done by figuring out what the options where.

sub ChatHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $login    = shift;					# Configuration tags	

	my $option;						# Options handle
	my $rjava;						# Formated javascript
	my $rchat;						# Formatted chat preferences
	my $rformat;						# Formatted format options
	my $rprivacy;						# Privacy Options


	#####################
	# Program area

	# Create instance
	$option   = new eRealms::Option();

	# Verify options received
	$rjava    = $option->FindJavascript($login{'OptJavascript'}, $param->{'CJAVA'});
	$rchat    = $option->FindChatPrefs($login{'OptChatPref'}, $param->{'CCHAT'});
	$rformat  = $option->FindFormatting($login{'OptFormat'}, $param->{'CFORMAT'});
	$rprivacy = $option->FindPrivacy($login{'OptPrivacy'}, $param->{'CPRIVACY'});

	# Evaluate
	eval
	{
		# Javascript options
		if ($rjava ne 'default')
		{
			# Update database entry
			$database->Write("UPDATE Puppeteer
				SET   PuppeteerJavascript=?
				WHERE PuppeteerLogin=?",

			 $rjava,
			 $param->{'USER'}
			);
		}

		# Chatting options
		if ($rchat ne 'default')
		{
			# Update database entry
			$database->Write("UPDATE Puppeteer
				SET   PuppeteerChatPref=?
				WHERE PuppeteerLogin=?",

			 $rchat,
			 $param->{'USER'}
			);
		}

		# Formatting options
		if ($rformat ne 'default')
		{
			# Update database entry
			$database->Write("UPDATE Puppeteer
				SET   PuppeteerFormatting=?
				WHERE PuppeteerLogin=?",

			 $rformat,
			 $param->{'USER'}
			);
		}

		# Privacy options
		if ($rprivacy ne 'default')
		{
			# Update database entry
			$database->Write("UPDATE Puppeteer
				SET   PuppeteerPrivacy=?
				WHERE PuppeteerLogin=?",

			 $rprivacy,
			 $param->{'USER'}
			);
		}

		# Commit changes
		$database->Commit();
	};

	# If block failed
	if ($@)
	{
		# Issue warning
		warn("Transaction aborted: $@");

		# Undo incomplete changes
		$database->Rollback();
	}
}


#####################
# Check Data
#  
# Arbitrary sub-routine designed to be used to check validity of data.  Made
# to hide ugliness of checks also centralize adapting of checks.

sub CheckData
{
	#####################
	# Data members
	my $value    = shift;					# Value to check 
	my $choice   = shift;					# Choice of the matter
	my $database = shift;					# Possible database handle
	my $user     = shift;					# User


	#####################
	# Program area

	# Password Check
	if ($choice == 0)
	{
		# Retreive Password
		my ($crypt) = $database->DataGet("SELECT PuppeteerPassword
			FROM  Puppeteer
			WHERE PuppeteerLogin=?", $user);	


		# Verify existence of user and validity of password
		if ((defined($crypt)) && ($crypt eq crypt(substr($value, 0, 8), $crypt)))
		{ return 1; }					# Return true
	}

	# Length
	elsif ($choice == 1)
	{
		# Lenght of a minumum of 3 characters
		if (length($value) > 3)
		{ return 1; }					# Return true
	}

	# E-Mail
	elsif ($choice == 2)
	{
		# Verify for legality
		if ($value =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s)
		{ return 1; }					# Return true
	}

	# HTTP
	elsif ($choice == 3)
	{
		# Verify for legality
		if ($value =~ /^http\:\/\//s)
		{ return 1; }					# Return true
	}

	# Numeric
	elsif ($choice == 4)
	{
		# Verify for legality
		if ($value =~ /^\d+$/s)
		{ return 1; }					# Return true
	}

	# If all else fails
	return 0; 
}

# Alias for Password validity
sub CheckDataPassword
{ CheckData($_[2], 0, $_[0], $_[1]); }

# Alias for Lenght
sub CheckDataLenght
{ CheckData(shift, 1); }

# Alias for E-Eail validity
sub CheckDataEMail
{ CheckData(shift, 2); }

# Alias for URL addresses
sub CheckDataAddress
{ CheckData(shift, 3); }

# Alias for Numeric filter
sub CheckDataNumeric
{ CheckData(shift, 4); }


#####################
# Contact Handler
#
# This sub-routine will allow the user to add in information about certain 
# contact information.    This is a necessary component for the enhaced gummy
# functionality.

sub ContactHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Configuration hash

	my $res;						# Search results
	my $table;						# Table handler
	my $statement;						# Search statements

	my $code;						# Title code
	my $info;						# Parameter date
	my $name;						# Parameter name
	my $type;						# Contact type
	my $user;						# Username

	my %user;						# User results


	#####################
	# Program area	
	
	# Initial handle
	$table = new eRealms::Table($cgi);

	# Initial assignment
	$type = 'None';
	$user = $param->{'USER'};


	# Initial user search and gather
	$database->Pull(\$statement, "SELECT
		  ContactName AS \"ContactName\", 
		  ContactData AS \"ContactData\"
		FROM PuppeteerContact
		WHERE PuppeteerLogin=?",

	 $param->{'USER'}
	);

	# Loop and gather
	while ($res = $statement->fetchrow_hashref())
	{
		# Store in hash for quick reference
		$user{$res->{'ContactName'}} = $res->{'ContactData'};
	}

	# End query
	$statement->finish();



	# Main operation
	# Prepare and execute
	$database->Pull(\$statement, "SELECT
		 ContactName   AS \"ContactName\",
		 ContactType   AS \"ContactType\",
		 ContactFilter AS \"ContactFilter\"
		FROM Contact
		ORDER BY ContactType, ContactName");


	# Information display
	# Title
	$tmpl->ShowMain($config->{'TagContactTitle'});


	# Form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), PROCESS=>'True');

	# Table top
	$table->MakeTop();

	# Loop and handle
	while ($res = $statement->fetchrow_hashref())
	{
		# Adjust name
		$name =  $res->{'ContactName'};
		$name =~ s/\W//gs;

		# Initial assigment
		$info =  $param->{$name};

		# Cleanup
		$param->Cleanup($name);


		# Ensure initial form displayed
		if (defined($param->{'PROCESS'}))
		{
			# Information check
		 	if (defined($info))	
			{
				# Email based search
				if ($res->{'ContactFilter'} eq 'email')
				{ $info = CheckDataEMail($info) ? $info : undef; }

				# Web address confirm
				elsif ($res->{'ContactFilter'} eq 'http')
				{ $info = CheckDataAddress($info) ? $info : undef; }

				# Numeric based acceptance
				elsif ($res->{'ContactFilter'} eq 'numeric')
				{ $info = CheckDataNumeric($info) ? $info : undef; }

				# Default filter
				else
				{ $info = CheckDataLenght($info) ? $info : undef; }
			}


			# Determine need
			# Creation
			if ((defined($info))
			 && (!defined($user{$res->{'ContactName'}})))
			{
				# Create parameter
				$database->Quick("INSERT INTO PuppeteerContact
					(ContactName,
					 ContactData,
					 PuppeteerLogin)
					VALUES(?,?,?)",

				 $res->{'ContactName'},
				 $info,
				 $user
				);
			}

			# Removal
			elsif ((!defined($info))
			 && (defined($user{$res->{'ContactName'}}))) 
			{
				# Remove parameter
				$database->Quick("DELETE FROM PuppeteerContact
					WHERE ContactName=?
					  AND PuppeteerLogin=?",

				 $res->{'ContactName'},
				 $user
				);
			}

			# Modify
			elsif ((defined($info))
			 && (defined($user{$res->{'ContactName'}}))
			 && ($info ne $user{$res->{'ContactName'}}))
			{
				# Modify parameter
				$database->Quick("UPDATE PuppeteerContact
					SET ContactData=?
					WHERE ContactName=?
					  AND PuppeteerLogin=?",

				 $info,
				 $res->{'ContactName'},
				 $user
				);
			}
		}

		# Initial pickup
		else
		{
			# Assign current value
			$info = $user{$res->{'ContactName'}};
		}

		# Final assignment
		$info = (defined($info)) ? $info : '';


		# Information display
		# Ensure defined
		$res->{'ContactType'} = (defined($res->{'ContactType'})) ? $res->{'ContactType'} : 'None';

		# Display section
		if (ucfirst($res->{'ContactType'}) ne $type)
		{
			# Massage
			$type =  ucfirst($res->{'ContactType'});

			# Display
			$table->MakeBlank();
			$table->MakeSingle($tmpl->PassSub($type));
		}

		# Display widget
		$table->MakeValid(
			"<B>$res->{ContactName}<B>",
			$cgi->textfield($name, $info, 35, 200)
		 );
	}

	# Finish query
	$statement->finish(); 

	# End table
	$table->MakeBlank();
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->submit($config->{'TxtResetButton'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form();
}


#####################
# Information Handler
#
# Will take the parameters and depending on information received can change the
# passwords and email.  The rest is considered non-essential.

sub InformationHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash
	
	my $audit;						# Auditing handle
	my $choice   = 0;					# Choice of action


	#####################
	# Program area	
	
	# Primary security check
	# Password Validity
	if (CheckDataPassword($database, $param->{'USER'}, $param->{'OLDPASS'}))
	{ 
		# Increment by one
		$choice++; 
	}
	else
	{ 
		# Display error message
		print "$config->{ErrPassword}\n"; 
	}


	# Password change
	if (defined($param->{'ONEPASS'}))
	{
		# Valid new passwords
		if ((CheckDataLenght($param->{'ONEPASS'})) && ($param->{'ONEPASS'} eq $param->{'TWOPASS'}) )
		{
			# Inrement by one 
			$choice++; 
		}
		else
		{
			# Display error message 
			print "$config->{ErrMismatch}\n"; 
		}
	}	

	# E-Mail change
	elsif (defined($param->{'EMAIL'}))
	{
		# Valid new E-Mail
		if (CheckDataEMail($param->{'EMAIL'}))
		{	
			# Inrement by one 
			$choice++; 
		}
		else
		{ 
			# Display error message
			print "$config->{ErrEMail}\n"; 
		}
	}

	# If both tests pass, change data
	if ($choice == 2)
	{
		# Auditing handle
		$audit = new eRealms::Audit($database, $cgi, $param);

		# Evaluate change
		eval
		{
			# Password change
			if ( defined($param->{'ONEPASS'}) )
			{
				# Create random string
				my $rand = md5_hex(rand());
				$rand = substr($rand, 0, 14);			

				# Insert new password
				$database->Write("UPDATE Puppeteer 
					SET   PuppeteerPassword=? 
					WHERE PuppeteerLogin=?", 

				 crypt(substr($param->{'ONEPASS'}, 0, 8), $rand), 
			 	 $param->{'USER'}
				);

				# Record
				$audit->PasswordChange($param->{'USER'});
			}
		
			# Email change
			elsif ( defined($param->{'EMAIL'}) )
			{
				# Insert new password
				$database->Write("UPDATE Puppeteer 
					SET PuppeteerEMail=? 
					WHERE PuppeteerLogin=?", 

				 $param->{'EMAIL'}, 
				 $param->{'USER'}
				);

				# Record
				$audit->EmailChange($param->{'USER'} . ':' . $param->{'EMAIL'});
			}


			# Commit changes
			$database->Commit();
		};

		# If block failed
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback();
		}


		# Notify of Success
		print "$config->{TagConfSuccess}\n";
	}
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Configuration hash
	my $login    = shift;					# Login hash

	my $table    = new eRealms::Table($cgi);		# Table handler
	my $url      = $cgi->url(-absolute=>1);			# Self referencing URL

	my $active;						# Active handles
	my $inactive;						# Inactive handles

	my @active;						# Active handles
	my @inactive;						# Inactive handles
	
	my %args;						# Arguments
	my %login;						# Login hash
	

	#####################
	# Program area	
	
	# Link with hashes
	$database->GetHashLogin(\%login);

	# Define base arguments
	%args = (
		USER  => $param->{'USER'},
		CRYPT => $param->{'CRYPT'}
	 );


	# Generate Inlines
	my ($icntc) = $param->EmbedInline(%args, CONTACT  => 'True');
	my ($imail) = $param->EmbedInline(%args, CMAIL    => 'True');
	my ($ipass) = $param->EmbedInline(%args, PASSWORD => 'True');
	my ($ipref) = $param->EmbedInline(%args);

	
	# Create Inline links
	print "<CENTER><A HREF=\"$url$ipref\">$config->{TagPrefs}</A>\n",
	      "<A HREF=\"$url$icntc\">$config->{TagContact}</A>\n",
	      "<A HREF=\"$url$imail\">$config->{TagEmail}</A>\n",
	      "<A HREF=\"$url$ipass\">$config->{TagPassword}</A></CENTER>\n<BR><BR>\n";

	# Password change
	if (defined($param->{'PASSWORD'}))
	{
		# Display of information
		# Title
		$tmpl->ShowMain($config->{'TagConfPassword'});

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), 'PASSWORD', 'True'), "\n";

		# Password section
		$table->MakeTop();
		$table->MakeValid(
		 $tmpl->PassSub($login{'TagPasswordOld'}),
		 $cgi->password_field('OLDPASS', '', 10, 15));
		$table->MakeValid(
		 $tmpl->PassSub($login{'TagPassword'}),
		 $cgi->password_field('ONEPASS', '', 10, 15));
		$table->MakeValid(
		 $tmpl->PassSub($login{'TagPasswordRepeat'}),
		 $cgi->password_field('TWOPASS', '', 10, 15));
		$table->MakeBlank();
	
		# Submit and bottom
		$table->MakeValid('&nbsp;', $cgi->submit($login{'TxtSubmitButtonSub'}));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}

	# Email address change
	elsif (defined($param->{'CMAIL'}))
	{
		# Pull Email
		my ($email) = $database->DataGetPuppeteerEmail($param->{'USER'});


		# Display of information
		# Title
		$tmpl->ShowMain($config->{'TagConfEMail'});

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), 'CMAIL', 'True');


		# Email section
		$table->MakeTop();
		$table->MakeValid(
		 $tmpl->PassSub($login{'TagPassword'}),
		 $cgi->password_field('OLDPASS', '', 10, 15));
		$table->MakeValid(
		 $tmpl->PassSub($login{'TagEMail'}),
		 $cgi->textfield('EMAIL', $email, 30, 45));
		$table->MakeBlank();

		# Submit and bottom
		$table->MakeValid('&nbsp;', $cgi->submit($login{'TxtSubmitButtonSub'}));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}

	# Contact information
	elsif (defined($param->{'CONTACT'}))
	{ 
		# Display and handle information
		ContactHandler($database, $cgi, $param, $tmpl, \%config);
 	}

	# Chat prefernces
	else
	{
		my $option;					# Option handler
		my $rjava;					# Formated javascript
		my $rchat;					# Formatted chat preferences
		my $rformat;					# Formatted format options
		my $rprivacy;					# Privacy options

		my @rjava;					# Javascript options
		my @rchat;					# Chat options
		my @rformat;					# Formatting options
		my @rprivacy;					# Privacy options


		# Split data into arrays
		$option = new eRealms::Option();
		$option->Split(\@rjava, $login{'OptJavascript'});
		$option->Split(\@rchat, $login{'OptChatPref'});
		$option->Split(\@rformat, $login{'OptFormat'});
		$option->Split(\@rprivacy, $login{'OptPrivacy'});

		# Data check
		$param->{'CJAVA'}    = defined($param->{'CJAVA'})    ? $param->{'CJAVA'}    : $rjava[2];
		$param->{'CCHAT'}    = defined($param->{'CCHAT'})    ? $param->{'CCHAT'}    : $rchat[4];
		$param->{'CFORMAT'}  = defined($param->{'CFORMAT'})  ? $param->{'CFORMAT'}  : $rformat[3];
		$param->{'CPRIVACY'} = defined($param->{'CPRIVACY'}) ? $param->{'CPRIVACY'} : $rprivacy[2];


		# Format options nicely
		$rjava    = $cgi->radio_group('CJAVA',    \@rjava,    $param->{'CJAVA'},    'true');
		$rchat    = $cgi->radio_group('CCHAT',    \@rchat,    $param->{'CCHAT'},    'true');
		$rformat  = $cgi->radio_group('CFORMAT',  \@rformat,  $param->{'CFORMAT'},  'true');
		$rprivacy = $cgi->radio_group('CPRIVACY', \@rprivacy, $param->{'CPRIVACY'}, 'true');


		# Display of information
		# Title
		$tmpl->ShowMain($config->{'TagConfPrefs'});

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), SETPREFS=>'TRUE');

		# Display of options
		$table->MakeTop();

		# Javascript
		$table->MakeSingle($tmpl->PassSub($login->{'TagOptJavacript'}));
		$table->MakeSingle($rjava);
		$table->MakeBlank();

		# Chat options
		$table->MakeSingle($tmpl->PassSub($login->{'TagOptChatPref'}));
		$table->MakeSingle($rchat);
		$table->MakeBlank();

		# Formating options
		$table->MakeSingle($tmpl->PassSub($login->{'TagOptFormat'}));
		$table->MakeSingle($rformat);
		$table->MakeBlank();
		
		# Privacy options
		$table->MakeSingle($tmpl->PassSub($login->{'TagOptPrivacy'}));
		$table->MakeSingle($rprivacy);
		$table->MakeBlank();

		# Submit and bottom
		$table->MakeSingle($cgi->submit($login{'TxtSubmitButtonSub'}));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}
}
